import 'package:json_annotation/json_annotation.dart';
import 'package:lori_sdk/data/models/product/product.dart';

part 'skuAttribute.g.dart';

//Sku Attribute object returned from the API
@JsonSerializable()
class SkuSpecificAttribute {
  SkuSpecificAttribute();

  String id;
  String skuId;
  SkuAttributeType type;
  String title;
  String value;
  String label;
  String order;

  factory SkuSpecificAttribute.fromJson(Map<String, dynamic> json) => _$SkuSpecificAttributeFromJson(json);

  Map<String, dynamic> toJson() => _$SkuSpecificAttributeToJson(this);
}

//Sku Attribute available type
enum SkuAttributeType {
  @JsonValue(0)
  text,

  @JsonValue(1)
  color,

  @JsonValue(2)
  picker
}

//Types created to simplify the UI construction
class SkuAttribute {
  String title;
  SkuAttributeType type;
  String order;
  List<SkuAttributeValue> values;
}

class SkuAttributeValue {
  String label;
  String value;
  bool isAvailable = true;

  SkuAttributeValue({this.label, this.value});

  @override
  String toString() {
    if (this.label != null && this.label.isNotEmpty) {
      return this.label;
    }
    else {
      return this.value;
    }
  }

  @override
  bool operator ==(Object other) => identical(this, other) ||
      other is SkuAttributeValue &&
          runtimeType == other.runtimeType &&
          this.label == other.label &&
          this.value == other.value;

  @override
  int get hashCode => label.hashCode ^ value.hashCode;
}


extension SkuAttributesListHelper on Iterable<Sku> {
  ///Retrieve all the existing attributes *values* along all skus
  ///
  /// This method will return all availables attributes, organized by title
  ///
  /// There are NO assurance that all combinations are available
  List<SkuAttribute> availableAttributes() {
    var skuList = this;

    //Get the title of the attributes, ordered by the attribute order
    var firstSkuAttrs = skuList.first.attributes;
    firstSkuAttrs.sort((la, ra) => la.order.compareTo(ra.order));
    var attrTitles = firstSkuAttrs.map((a) => a.title).toList();

    skuList.forEach((sku) {
      var skuAttrs = sku.attributes;
      skuAttrs.sort((la, ra) => la.order.compareTo(ra.order));
      var skuAttrTitles = skuAttrs.map((a) => a.label).toList();

      if (attrTitles != skuAttrTitles) {
        //If any sku has a different sku attributes than the others we can't use the sku attributes
        return null;
      }
    });

    //Get ALL attributes and values available
    var allAttributes = skuList.expand((sku) => sku.attributes);

    //Organize the attributes by title
    return allAttributes.organizedAttributes();
  }

  ///Return all the available values for an specific SKU based on a filter
  ///
  /// This method should be use to update the availability of SKU attributes after the user has selected one
  List<SkuAttributeValue> availableValuesForSkuAttributeWithTitle(String title, {Map<String, String> filters = null}) {
    //Filter the sku list according to the specified filter
    var filteredSkuList = this;
    if (filters != null) {
      for (var key in filters.keys) {
        filteredSkuList = filteredSkuList.where((sku) {
          return sku.attributes.where((a) {
            return a.title == key && a.value == filters[key];
          }).length > 0;
        }).toList();
      }
    }

    //Get all attributes with the specified title
    var allAttributes = filteredSkuList.expand((sku) => sku.attributes).where((at) => at.title == title);

    //Return the values
    return allAttributes.attributeWithTitle(title).values;
  }
}

extension SkuAttributeValuesHelper on Iterable<SkuSpecificAttribute> {
  /// "Organize" an array of SkuSpecificAttributes, grouping the sku values by title
  List<SkuAttribute> organizedAttributes() {
    //Get all titles available
    var allTitles = this.map((attr) => attr.title).toSet().toList();

    //Create the array to hold the available attributes
    List<SkuAttribute> organizedAttributes = List<SkuAttribute>();

    //Loop through all the labels
    allTitles.forEach((title) {
      var attributeValues = this.attributeWithTitle(title);

      organizedAttributes.add(attributeValues);
    });

    return organizedAttributes;
  }

  /// "Organize" an array of SkuSpecificAttributed, but considering only one title
  SkuAttribute attributeWithTitle(String title) {
    //Get all attributes with this label
    var attributesWithTitle = this.where((a) => a.title == title);

    //Create the wrapper
    var attributeValues = SkuAttribute();
    attributeValues.title = title;
    //The type should be equal on all attributes with the same label, so pick the first
    attributeValues.type = attributesWithTitle.first.type;
    //The same is valid to order
    attributeValues.order = attributesWithTitle.first.order;
    //Create an set with all the values
    attributeValues.values = attributesWithTitle.map((a) => SkuAttributeValue(label: a.label, value: a.value)).toList().toSet().toList();
    //Sort the values
    attributeValues.values.sort((lv, rv) => lv.toString().compareTo(rv.toString()));

    return attributeValues;
  }
}